@@ -1,21 +1,22 @@
 //! Erlang top-level components such as attributes, directives or declarations.
-use crate::format::{self, Format};
+use crate::format::{Format, Formatter, Indent, Newline};
 use crate::items::atoms::{
     CallbackAtom, DefineAtom, IncludeAtom, IncludeLibAtom, OpaqueAtom, RecordAtom, SpecAtom,
     TypeAtom,
 };
-use crate::items::expressions::functions::FunctionClause;
-use crate::items::expressions::Expr;
-use crate::items::generics::{
-    Args, Clauses, Either, Maybe, Params, Parenthesized, TupleLike, WithArrow, WithGuard,
+use crate::items::components::{
+    Clauses, CommaDelimiter, Either, Element, Maybe, Never, NonEmptyItems, Null, Params,
+    Parenthesized, TupleLike, WithArrow, WithGuard,
 };
+use crate::items::expressions::components::FunctionClause;
 use crate::items::keywords::IfKeyword;
 use crate::items::macros::{MacroName, MacroReplacement};
 use crate::items::symbols::{
     CloseParenSymbol, ColonSymbol, CommaSymbol, DotSymbol, DoubleColonSymbol, HyphenSymbol,
     MatchSymbol, OpenParenSymbol,
 };
-use crate::items::tokens::{AtomToken, StringToken, Token, VariableToken};
+use crate::items::tokens::{AtomToken, LexicalToken, StringToken, VariableToken};
+use crate::items::Expr;
 use crate::items::Type;
 use crate::parse::Parse;
 use crate::span::Span;
@@ -36,47 +37,30 @@ pub(super) enum Form {
 ///
 /// - $NAME: [AtomToken]
 /// - $FIELD: [AtomToken] (`=` [Expr])? (`::` [Type])? `,`?
+#[derive(Debug, Clone, Span, Parse, Format)]
+pub struct RecordDecl(AttrLike<RecordAtom, RecordDeclValue>);
+
 #[derive(Debug, Clone, Span, Parse)]
-pub struct RecordDecl {
-    hyphen: HyphenSymbol,
-    record: RecordAtom,
-    open: OpenParenSymbol,
+struct RecordDeclValue {
     name: AtomToken,
     comma: CommaSymbol,
     fields: TupleLike<RecordField>,
-    close: CloseParenSymbol,
-    dot: DotSymbol,
 }
 
-impl Format for RecordDecl {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        self.hyphen.format(fmt)?;
-        self.record.format(fmt)?;
-        self.open.format(fmt)?;
-        fmt.subregion().current_column_as_indent().enter(|fmt| {
-            self.name.format(fmt)?;
-            self.comma.format(fmt)?;
-            fmt.write_space()?;
-            fmt.subregion()
-                .forbid_multi_line()
-                .forbid_too_long_line()
-                .reset_trailing_columns(2) // ")."
-                .check_trailing_columns(true)
-                .enter(|fmt| self.fields.format(fmt))
-                .or_else(|_| {
-                    fmt.write_newline()?;
-                    fmt.subregion()
-                        .reset_trailing_columns(2) // ")."
-                        .enter(|fmt| self.fields.format(fmt))
-                })
-        })?;
-        self.close.format(fmt)?;
-        self.dot.format(fmt)?;
-        Ok(())
+impl Format for RecordDeclValue {
+    fn format(&self, fmt: &mut Formatter) {
+        fmt.subregion(Indent::CurrentColumn, Newline::Never, |fmt| {
+            self.name.format(fmt);
+            self.comma.format(fmt);
+            fmt.add_space();
+            fmt.subregion(Indent::inherit(), Newline::IfTooLongOrMultiLine, |fmt| {
+                self.fields.format(fmt)
+            });
+        });
     }
 }
 
-#[derive(Debug, Clone, Span, Parse)]
+#[derive(Debug, Clone, Span, Parse, Element)]
 struct RecordField {
     name: AtomToken,
     default: Maybe<(MatchSymbol, Expr)>,
@@ -84,21 +68,20 @@ struct RecordField {
 }
 
 impl Format for RecordField {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        self.name.format(fmt)?;
+    fn format(&self, fmt: &mut Formatter) {
+        self.name.format(fmt);
         if let Some((x, y)) = self.default.get() {
-            fmt.write_space()?;
-            x.format(fmt)?;
-            fmt.write_space()?;
-            y.format(fmt)?;
+            fmt.add_space();
+            x.format(fmt);
+            fmt.add_space();
+            y.format(fmt);
         }
         if let Some((x, y)) = self.r#type.get() {
-            fmt.write_space()?;
-            x.format(fmt)?;
-            fmt.write_space()?;
-            y.format(fmt)?;
+            fmt.add_space();
+            x.format(fmt);
+            fmt.add_space();
+            y.format(fmt);
         }
-        Ok(())
     }
 }
 
@@ -110,12 +93,9 @@ impl Format for RecordField {
 ///
 /// Note that the parenthesized notation like `-type(foo() :: bar()).` is also acceptable
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct TypeDecl {
-    hyphen: HyphenSymbol,
-    kind: Either<TypeAtom, OpaqueAtom>,
-    item: Either<TypeDeclItem, Parenthesized<TypeDeclItem>>,
-    dot: DotSymbol,
-}
+pub struct TypeDecl(AttrLike<TypeDeclName, TypeDeclItem>);
+
+type TypeDeclName = Either<TypeAtom, OpaqueAtom>;
 
 #[derive(Debug, Clone, Span, Parse)]
 struct TypeDeclItem {
@@ -126,35 +106,17 @@ struct TypeDeclItem {
 }
 
 impl Format for TypeDeclItem {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        if fmt.last_char() != '(' {
-            fmt.write_space()?;
-        }
-
-        fmt.subregion().current_column_as_indent().enter(|fmt| {
-            self.name.format(fmt)?;
-            fmt.subregion()
-                .reset_trailing_columns(3) // " ::"
-                .enter(|fmt| self.params.format(fmt))?;
-            fmt.write_space()?;
-            self.delimiter.format(fmt)?;
-            fmt.write_space()?;
-            fmt.subregion()
-                .current_column_as_indent()
-                .forbid_too_long_line()
-                .trailing_columns(1) // "."
-                .check_trailing_columns(true)
-                .enter(|fmt| self.r#type.format(fmt))
-                .or_else(|_| {
-                    fmt.write_newline()?;
-                    fmt.subregion()
-                        .trailing_columns(1) // "."
-                        .indent_offset(2)
-                        .enter(|fmt| self.r#type.format(fmt))
-                })?;
-            Ok(())
-        })?;
-        Ok(())
+    fn format(&self, fmt: &mut Formatter) {
+        fmt.subregion(Indent::CurrentColumn, Newline::Never, |fmt| {
+            self.name.format(fmt);
+            self.params.format(fmt);
+            fmt.add_space();
+            self.delimiter.format(fmt);
+            fmt.add_space();
+            fmt.subregion(Indent::Offset(2), Newline::IfTooLong, |fmt| {
+                self.r#type.format(fmt)
+            });
+        });
     }
 }
 
@@ -165,25 +127,10 @@ impl Format for TypeDeclItem {
 /// - $RETURN: [Type]
 ///
 /// Note that the parenthesized notation like `-spec(foo() -> bar()).` is also acceptable
-#[derive(Debug, Clone, Span, Parse)]
-pub struct FunSpec {
-    hyphen: HyphenSymbol,
-    kind: Either<SpecAtom, CallbackAtom>,
-    item: Either<FunSpecItem, Parenthesized<FunSpecItem>>,
-    dot: DotSymbol,
-}
+#[derive(Debug, Clone, Span, Parse, Format)]
+pub struct FunSpec(AttrLike<FunSpecName, FunSpecItem>);
 
-impl Format for FunSpec {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        self.hyphen.format(fmt)?;
-        self.kind.format(fmt)?;
-        fmt.subregion()
-            .reset_trailing_columns(1) // "."
-            .enter(|fmt| self.item.format(fmt))?;
-        self.dot.format(fmt)?;
-        Ok(())
-    }
-}
+type FunSpecName = Either<SpecAtom, CallbackAtom>;
 
 #[derive(Debug, Clone, Span, Parse)]
 struct FunSpecItem {
@@ -193,45 +140,29 @@ struct FunSpecItem {
 }
 
 impl Format for FunSpecItem {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        if fmt.last_char() != '(' {
-            fmt.write_space()?;
-        }
-
-        fmt.subregion().current_column_as_indent().enter(|fmt| {
-            self.module_name.format(fmt)?;
-            self.function_name.format(fmt)?;
-            self.clauses.format(fmt)
-        })?;
-        Ok(())
+    fn format(&self, fmt: &mut Formatter) {
+        fmt.subregion(Indent::CurrentColumn, Newline::Never, |fmt| {
+            self.module_name.format(fmt);
+            self.function_name.format(fmt);
+            self.clauses.format(fmt);
+        });
     }
 }
 
 #[derive(Debug, Clone, Span, Parse)]
 struct SpecClause {
     params: WithArrow<Params<Type>>,
-    r#return: WithGuard<Type, Type, CommaSymbol>,
+    r#return: WithGuard<Type, Type, CommaDelimiter>,
 }
 
 impl Format for SpecClause {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        fmt.subregion()
-            .clear_trailing_columns(true)
-            .enter(|fmt| self.params.format(fmt))?;
-
-        fmt.subregion()
-            .forbid_multi_line()
-            .forbid_too_long_line()
-            .check_trailing_columns(true)
-            .enter(|fmt| self.r#return.format(fmt))
-            .or_else(|_| {
-                fmt.write_newline()?;
-                fmt.subregion()
-                    .parent_indent()
-                    .indent_offset(4)
-                    .enter(|fmt| self.r#return.format(fmt))
-            })?;
-        Ok(())
+    fn format(&self, fmt: &mut Formatter) {
+        self.params.format(fmt);
+        fmt.subregion(
+            Indent::ParentOffset(4),
+            Newline::IfTooLongOrMultiLine,
+            |fmt| self.r#return.format(fmt),
+        );
     }
 }
 
@@ -241,46 +172,40 @@ impl Format for SpecClause {
 /// - $PARAM: [Expr]
 /// - $GUARD: ([Expr] (`,` | `;`)?)+
 /// - $BODY: ([Expr] `,`?)+
-#[derive(Debug, Clone, Span, Parse)]
+#[derive(Debug, Clone, Span, Parse, Format)]
 pub struct FunDecl {
     clauses: Clauses<FunctionClause<AtomToken>>,
     dot: DotSymbol,
 }
 
-impl Format for FunDecl {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        fmt.subregion()
-            .reset_trailing_columns(1) // "."
-            .enter(|fmt| self.clauses.format(fmt))?;
-        self.dot.format(fmt)?;
-        Ok(())
-    }
-}
-
 /// `-` `$NAME` `$ARGS`? `.`
 ///
 /// - $NAME: [AtomToken] | `if`
 /// - $ARGS: `(` (`$ARG` `,`?)* `)`
 /// - $ARG: [Expr]
+#[derive(Debug, Clone, Span, Parse, Format)]
+pub struct Attr(AttrLike<AttrName, AttrValue, Null>);
+
+type AttrName = Either<AtomToken, IfKeyword>;
+type AttrValue = NonEmptyItems<Expr>;
+
 #[derive(Debug, Clone, Span, Parse)]
-pub struct Attr {
+struct AttrLike<Name, Value, Empty = Never> {
     hyphen: HyphenSymbol,
-    name: Either<AtomToken, IfKeyword>,
-    items: Maybe<Args<Expr>>,
+    name: Name,
+    value: Either<Parenthesized<Value>, Either<Value, Empty>>,
     dot: DotSymbol,
 }
 
-impl Format for Attr {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        self.hyphen.format(fmt)?;
-        self.name.format(fmt)?;
-        if let Some(items) = self.items.get() {
-            fmt.subregion()
-                .reset_trailing_columns(1) // "."
-                .enter(|fmt| items.format(fmt))?;
+impl<Name: Format, Value: Format, Empty: Format> Format for AttrLike<Name, Value, Empty> {
+    fn format(&self, fmt: &mut Formatter) {
+        self.hyphen.format(fmt);
+        self.name.format(fmt);
+        if matches!(self.value, Either::B(Either::A(_))) {
+            fmt.add_space();
         }
-        self.dot.format(fmt)?;
-        Ok(())
+        self.value.format(fmt);
+        self.dot.format(fmt);
     }
 }
 
@@ -288,7 +213,7 @@ impl Format for Attr {
 ///
 /// - $NAME: [AtomToken] | [VariableToken]
 /// - $VARS: `(` ([VariableToken] `,`?)* `)`
-/// - $REPLACEMENT: [Token]
+/// - $REPLACEMENT: [LexicalToken]
 #[derive(Debug, Clone, Span, Parse)]
 pub struct DefineDirective {
     hyphen: HyphenSymbol,
@@ -311,37 +236,27 @@ impl DefineDirective {
         self.variables.get().map(|x| x.get())
     }
 
-    pub fn replacement(&self) -> &[Token] {
+    pub fn replacement(&self) -> &[LexicalToken] {
         self.replacement.tokens()
     }
 }
 
 impl Format for DefineDirective {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        self.hyphen.format(fmt)?;
-        self.define.format(fmt)?;
-        self.open.format(fmt)?;
-        fmt.subregion().current_column_as_indent().enter(|fmt| {
-            self.macro_name.format(fmt)?;
-            self.variables.format(fmt)?;
-            self.comma.format(fmt)?;
-            fmt.write_space()?;
-            fmt.subregion()
-                .forbid_multi_line()
-                .forbid_too_long_line()
-                .reset_trailing_columns(2) // ")."
-                .check_trailing_columns(true)
-                .enter(|fmt| self.replacement.format(fmt))
-                .or_else(|_| {
-                    fmt.write_newline()?;
-                    fmt.subregion()
-                        .reset_trailing_columns(2) // ")."
-                        .enter(|fmt| self.replacement.format(fmt))
-                })
-        })?;
-        self.close.format(fmt)?;
-        self.dot.format(fmt)?;
-        Ok(())
+    fn format(&self, fmt: &mut Formatter) {
+        self.hyphen.format(fmt);
+        self.define.format(fmt);
+        self.open.format(fmt);
+        fmt.subregion(Indent::CurrentColumn, Newline::Never, |fmt| {
+            self.macro_name.format(fmt);
+            self.variables.format(fmt);
+            self.comma.format(fmt);
+            fmt.add_space();
+            fmt.subregion(Indent::inherit(), Newline::IfTooLongOrMultiLine, |fmt| {
+                self.replacement.format(fmt)
+            });
+        });
+        self.close.format(fmt);
+        self.dot.format(fmt);
     }
 }
 
@@ -363,42 +278,7 @@ impl IncludeDirective {
         self.file.value()
     }
 
-    pub fn get_include_path(&self, include_dirs: &[PathBuf]) -> Option<PathBuf> {
-        let path = self.get_var_substituted_path();
-        if matches!(self.include, Either::B(_)) && path.components().count() > 1 {
-            let app_name = if let std::path::Component::Normal(name) = path.components().next()? {
-                name.to_str()?
-            } else {
-                return None;
-            };
-            match crate::erl::erl_eval(&format!("code:lib_dir({})", app_name)) {
-                Err(e) => {
-                    log::warn!("{}", e);
-                    None
-                }
-                Ok(base_dir) => {
-                    let mut resolved_path = PathBuf::from(base_dir);
-                    resolved_path.extend(path.components().skip(1));
-                    log::debug!("Resolved include path: {:?}", resolved_path);
-                    Some(resolved_path)
-                }
-            }
-        } else if path.exists() {
-            log::debug!("Resolved include path: {:?}", path);
-            Some(path)
-        } else {
-            for dir in include_dirs {
-                let candidate_path = dir.join(&path);
-                if candidate_path.exists() {
-                    log::debug!("Resolved include path: {:?}", candidate_path);
-                    return Some(candidate_path);
-                }
-            }
-            None
-        }
-    }
-
-    fn get_var_substituted_path(&self) -> PathBuf {
+    pub fn var_substituted_path(&self) -> PathBuf {
         let path_str = self.file.value();
         let path: &Path = path_str.as_ref();
         if !path_str.starts_with('$') {
@@ -421,6 +301,40 @@ impl IncludeDirective {
         }
         expanded_path
     }
+
+    pub fn resolved_path(&self, include_dirs: &[PathBuf]) -> Option<PathBuf> {
+        let path = self.var_substituted_path();
+        if matches!(self.include, Either::B(_)) && path.components().count() > 1 {
+            let app_name = if let std::path::Component::Normal(name) = path.components().next()? {
+                name.to_str()?
+            } else {
+                return None;
+            };
+            match crate::erl::code_lib_dir(app_name) {
+                Err(e) => {
+                    log::warn!("{}", e);
+                    None
+                }
+                Ok(mut resolved_path) => {
+                    resolved_path.extend(path.components().skip(1));
+                    log::debug!("Resolved include path: {:?}", resolved_path);
+                    Some(resolved_path)
+                }
+            }
+        } else if path.exists() {
+            log::debug!("Resolved include path: {:?}", path);
+            Some(path)
+        } else {
+            for dir in include_dirs {
+                let candidate_path = dir.join(&path);
+                if candidate_path.exists() {
+                    log::debug!("Resolved include path: {:?}", candidate_path);
+                    return Some(candidate_path);
+                }
+            }
+            None
+        }
+    }
 }
 
 #[cfg(test)]
@@ -496,6 +410,9 @@ mod tests {
                         no_match],
                        [g/0,
                         h/0]})."},
+            indoc::indoc! {"
+            -export [foo/0,
+                     bar/1]."},
         ];
         for text in texts {
             crate::assert_format!(text, Form);
@@ -533,6 +450,11 @@ mod tests {
             foo(_, _) ->
                 baz."},
             indoc::indoc! {"
+            %---10---|%---20---|
+            foo(A)
+              when a, b; c ->
+                d."},
+            indoc::indoc! {"
             foo(A)
               when is_atom(A) ->
                 bar,
