@@ -1,11 +1,11 @@
-use crate::format::{self, Format, Formatter};
-use crate::items::expressions::Expr;
-use crate::items::generics::{Args, Either, Maybe};
+use crate::format::{Format, Formatter};
+use crate::items::components::{Args, Either, Maybe};
 use crate::items::symbols::{
     CloseParenSymbol, CommaSymbol, DotSymbol, OpenParenSymbol, QuestionSymbol,
 };
-use crate::items::tokens::{AtomToken, StringToken, Token, VariableToken};
-use crate::parse::{self, Parse, TokenStream};
+use crate::items::tokens::{AtomToken, LexicalToken, StringToken, VariableToken};
+use crate::items::Expr;
+use crate::parse::{self, Parse, ResumeParse, TokenStream};
 use crate::span::{Position, Span};
 use erl_tokenize::values::{Keyword, Symbol};
 use std::collections::HashMap;
@@ -13,42 +13,45 @@ use std::collections::HashMap;
 /// `?` `$NAME` (`(` (`$ARG` `,`?)* `)`)?
 ///
 /// - $NAME: [AtomToken] | [VariableToken]
-/// - $ARG: [Token]+
-#[derive(Debug, Clone, Span, Format)]
+/// - $ARG: [LexicalToken]+
+#[derive(Debug, Clone, Span, Parse, Format)]
 pub struct Macro {
     question: QuestionSymbol,
     name: MacroName,
-    args: Maybe<MacroArgs>,
+    args: Maybe<Args<MacroArg>>,
 }
 
-impl Macro {
-    // TODO: resume_parse
-    pub(crate) fn parse(
+impl ResumeParse<(QuestionSymbol, MacroName, bool)> for Macro {
+    fn resume_parse(
         ts: &mut TokenStream,
-        question: QuestionSymbol,
-        name: MacroName,
-        has_args: bool,
+        (question, name, has_args): (QuestionSymbol, MacroName, bool),
     ) -> parse::Result<Self> {
         if has_args {
             Ok(Self {
                 question,
                 name,
-                args: Maybe::from_item(ts.parse()?),
+                args: Maybe::some(ts.parse()?),
             })
         } else {
             Ok(Self {
                 question,
                 name,
-                args: Maybe::none(ts)?,
+                args: Maybe::parse_none(ts)?,
             })
         }
     }
+}
 
+impl Macro {
     pub fn arity(&self) -> Option<usize> {
         self.args.get().map(|x| x.get().len())
     }
 
-    pub fn expand(&self, variables: Option<Vec<String>>, replacement: Vec<Token>) -> Vec<Token> {
+    pub fn expand(
+        &self,
+        variables: Option<Vec<String>>,
+        replacement: Vec<LexicalToken>,
+    ) -> Vec<LexicalToken> {
         let args = if let (Some(vars), Some(vals)) = (&variables, self.args.get().map(|x| x.get()))
         {
             vars.iter()
@@ -63,26 +66,25 @@ impl Macro {
         let mut tokens = Vec::new();
         for token in replacement {
             match token {
-                Token::Variable(x) if do_stringify => {
+                LexicalToken::Variable(_) if do_stringify => {
                     let dummy =
-                        StringToken::new("EFMT_DUMMY", x.start_position(), x.end_position());
+                        StringToken::new("EFMT_DUMMY", self.start_position(), self.end_position());
                     tokens.push(dummy.into());
                 }
-                Token::Variable(x) if args.contains_key(x.value()) => {
+                LexicalToken::Variable(x) if args.contains_key(x.value()) => {
                     tokens.extend(args[x.value()].tokens().iter().cloned());
                 }
-                Token::Symbol(x) if x.value() == Symbol::DoubleQuestion => {
+                LexicalToken::Symbol(x) if x.value() == Symbol::DoubleQuestion => {
                     do_stringify = true;
                     continue;
                 }
-                token => {
+                mut token => {
+                    token.set_span(self);
                     tokens.push(token);
                 }
             }
             do_stringify = false;
         }
-        // TODO: delete(?)
-        tokens.iter_mut().for_each(|token| token.set_span(self));
         tokens
     }
 }
@@ -101,13 +103,12 @@ impl MacroName {
 
 #[derive(Debug, Clone)]
 pub(crate) struct MacroReplacement {
-    tokens: Vec<Token>,
-    expr: Option<Expr>, // The expression representation of `tokens` (for formatting)
+    tokens: Vec<LexicalToken>,
     start_position: Position,
 }
 
 impl MacroReplacement {
-    pub fn tokens(&self) -> &[Token] {
+    pub fn tokens(&self) -> &[LexicalToken] {
         &self.tokens
     }
 }
@@ -128,18 +129,14 @@ impl Span for MacroReplacement {
 
 impl Parse for MacroReplacement {
     fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
-        ts.enter_macro_replacement(|ts| {
-            let start_position = ts.current_whitespace_token()?.end_position();
-            let expr = ts
-                .peek::<(Expr, (CloseParenSymbol, DotSymbol))>()
-                .map(|(expr, _)| expr);
+        ts.with_macro_expand_disabled(|ts| {
+            let start_position = ts.next_token_start_position()?;
             let mut tokens = Vec::new();
             while ts.peek::<(CloseParenSymbol, DotSymbol)>().is_none() {
                 tokens.push(ts.parse()?);
             }
             Ok(Self {
                 tokens,
-                expr,
                 start_position,
             })
         })
@@ -147,66 +144,28 @@ impl Parse for MacroReplacement {
 }
 
 impl Format for MacroReplacement {
-    fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
-        if let Some(expr) = &self.expr {
-            expr.format(fmt)
-        } else {
-            fmt.write_text(self)
+    fn format(&self, fmt: &mut Formatter) {
+        if let Ok(expr) = fmt
+            .token_stream_mut()
+            .parse_tokens::<Expr>(self.tokens.clone())
+        {
+            if expr.end_position() == self.end_position() {
+                expr.format(fmt);
+                return;
+            }
         }
-    }
-}
 
-#[derive(Debug, Clone)]
-struct Empty;
-
-impl Span for Empty {
-    fn start_position(&self) -> Position {
-        unreachable!()
-    }
-
-    fn end_position(&self) -> Position {
-        unreachable!()
-    }
-}
-
-impl Parse for Empty {
-    fn parse(ts: &mut parse::TokenStream) -> parse::Result<Self> {
-        let token = ts.parse()?;
-        Err(parse::Error::unexpected_token(ts, token))
-    }
-}
-
-impl Format for Empty {
-    fn format(&self, _: &mut format::Formatter) -> format::Result<()> {
-        unreachable!()
-    }
-}
-
-#[derive(Debug, Clone, Span, Parse, Format)]
-enum MacroArgs {
-    Empty(Args<Empty>),
-    Args(Args<MacroArg>),
-}
-
-impl MacroArgs {
-    fn get(&self) -> &[MacroArg] {
-        match self {
-            Self::Empty(_) => &[],
-            Self::Args(x) => x.get(),
-        }
+        fmt.add_span(self);
     }
 }
 
 #[derive(Debug, Clone)]
 struct MacroArg {
-    tokens: Vec<Token>,
-
-    // TODO: Support macros in a macro arg
-    expr: Option<Expr>, // The expression representation of `tokens` (for formatting)
+    tokens: Vec<LexicalToken>,
 }
 
 impl MacroArg {
-    pub fn tokens(&self) -> &[Token] {
+    pub fn tokens(&self) -> &[LexicalToken] {
         &self.tokens
     }
 }
@@ -223,91 +182,57 @@ impl Span for MacroArg {
 
 impl Parse for MacroArg {
     fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
-        let mut expr = ts.peek::<Expr>();
-
-        #[derive(Debug, Default, PartialEq, Eq)]
-        struct Level {
-            paren: usize,
-            brace: usize,
-            square: usize,
-            bits: usize,
-            block: usize,
-        }
-
-        impl Level {
-            fn is_toplevel(&self) -> bool {
-                *self == Self::default()
-            }
+        if ts.peek::<Either<CommaSymbol, CloseParenSymbol>>().is_some() {
+            let token = ts.parse()?;
+            return Err(parse::Error::unexpected_token(ts, token));
         }
 
         let mut tokens = Vec::new();
-        let mut level = Level::default();
+        let mut level = 0;
         while tokens.is_empty()
-            || !level.is_toplevel()
+            || level > 0
             || ts.peek::<Either<CommaSymbol, CloseParenSymbol>>().is_none()
         {
-            let token: Token = ts.parse()?;
+            let token: LexicalToken = ts.parse()?;
             match &token {
-                Token::Symbol(x) => match x.value() {
-                    Symbol::OpenParen => {
-                        level.paren += 1;
+                LexicalToken::Symbol(x) => match x.value() {
+                    Symbol::OpenParen
+                    | Symbol::OpenBrace
+                    | Symbol::OpenSquare
+                    | Symbol::DoubleLeftAngle => {
+                        level += 1;
                     }
-                    Symbol::CloseParen => {
-                        if level.paren == 0 {
-                            todo!("{:?}", ts.current_position());
+                    Symbol::CloseParen
+                    | Symbol::CloseBrace
+                    | Symbol::CloseSquare
+                    | Symbol::DoubleRightAngle => {
+                        if level == 0 {
+                            return Err(parse::Error::unexpected_token(ts, token));
                         }
-                        level.paren -= 1;
-                    }
-                    Symbol::OpenBrace => {
-                        level.brace += 1;
-                    }
-                    Symbol::CloseBrace => {
-                        if level.brace == 0 {
-                            todo!("{:?}", ts.current_position());
-                        }
-                        level.brace -= 1;
-                    }
-                    Symbol::OpenSquare => {
-                        level.square += 1;
-                    }
-                    Symbol::CloseSquare => {
-                        if level.square == 0 {
-                            todo!("{:?}", ts.current_position());
-                        }
-
-                        level.square -= 1;
-                    }
-                    Symbol::DoubleLeftAngle => {
-                        level.bits += 1;
-                    }
-                    Symbol::DoubleRightAngle => {
-                        if level.bits == 0 {
-                            todo!("{:?}", ts.current_position());
-                        }
-                        level.bits -= 1;
+                        level -= 1;
                     }
                     _ => {}
                 },
-                Token::Keyword(x) => match x.value() {
+                LexicalToken::Keyword(x) => match x.value() {
                     Keyword::Begin
                     | Keyword::Try
                     | Keyword::Case
                     | Keyword::If
                     | Keyword::Receive => {
-                        level.block += 1;
+                        level += 1;
                     }
                     Keyword::Fun => {
                         if ts.peek::<OpenParenSymbol>().is_some()
-                            || ts.peek::<(Token, OpenParenSymbol)>().is_some()
+                            || ts.peek::<(LexicalToken, OpenParenSymbol)>().is_some()
                         {
-                            level.block += 1;
+                            level += 1;
                         }
                     }
                     Keyword::End => {
-                        if level.block == 0 {
-                            todo!("{:?}", ts.current_position());
+                        if level == 0 {
+                            return Err(parse::Error::unexpected_token(ts, token));
                         }
-                        level.block -= 1;
+                        level -= 1;
                     }
                     _ => {}
                 },
@@ -316,24 +241,23 @@ impl Parse for MacroArg {
             tokens.push(token);
         }
 
-        let is_expr = expr.as_ref().map_or(false, |x| {
-            x.end_position() == tokens[tokens.len() - 1].end_position()
-        });
-        if !is_expr {
-            expr = None;
-        }
-
-        Ok(Self { tokens, expr })
+        Ok(Self { tokens })
     }
 }
 
 impl Format for MacroArg {
-    fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
-        if let Some(expr) = &self.expr {
-            expr.format(fmt)
-        } else {
-            fmt.write_text(self)
+    fn format(&self, fmt: &mut Formatter) {
+        if let Ok(expr) = fmt
+            .token_stream_mut()
+            .parse_tokens::<Expr>(self.tokens.clone())
+        {
+            if expr.end_position() == self.end_position() {
+                expr.format(fmt);
+                return;
+            }
         }
+
+        fmt.add_span(self);
     }
 }
 
@@ -376,13 +300,13 @@ mod tests {
             -define(FOO_CLOSE,
                     )).
             foo(A) ->
-                ?FOO_OPEN A ?FOO_CLOSE.
+                ?FOO_OPEN A?FOO_CLOSE.
             "},
             indoc::indoc! {"
             -define(EMPTY, ).
 
-            ?EMPTY hello ?EMPTY () ->
-                ?EMPTY ?EMPTY world.
+            ?EMPTY hello?EMPTY() ->
+                ?EMPTY?EMPTY world.
             ?EMPTY"},
         ];
         for text in texts {
@@ -453,6 +377,22 @@ mod tests {
             foo() ->
                 ?FOO() + 10.
             "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(FOO,
+                    begin
+                        1,
+                        2
+                    end).
+            foo() ->
+                ?FOO.
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(FOO(X), X).
+            -define(BAR(),
+                    ?FOO(baz),).
+            "},
         ];
         for text in texts {
             crate::assert_format!(text, Module);
@@ -467,14 +407,16 @@ mod tests {
             -define(bar(A), A).
 
             main() ->
-                ?bar(?foo) c].
+                ?bar(?foo)
+                c].
             "},
             indoc::indoc! {"
             -define(foo, [],).
             -define(bar(A), A).
 
             main() ->
-                ?bar(?foo) [c].
+                ?bar(?foo)
+                [c].
             "},
             indoc::indoc! {"
             -define(a,
@@ -482,7 +424,8 @@ mod tests {
             -define(b(A), A).
 
             main() ->
-                ?b(?a a), c].
+                ?b(?a a)
+                , c].
             "},
         ];
         for text in texts {
@@ -529,7 +472,9 @@ mod tests {
             indoc::indoc! {"
             %---10---|%---20---|
             -define(BAR,
-                    ?FOO 1 end).
+                    ?FOO
+                        1
+                    end).
             -define(FOO, begin).
             baz() ->
                 ?BAR.
@@ -551,13 +496,23 @@ mod tests {
 
     #[test]
     fn circular_macro_works() {
-        let texts = [indoc::indoc! {"
+        let texts = [
+            indoc::indoc! {"
             %---10---|%---20---|
             -define(a, ?b).
             -define(b, ?a).
             foo() ->
                 ?a == ?b.
-            "}];
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(a, ?a).
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(a, ?a + ?a).
+            "},
+        ];
         for text in texts {
             crate::assert_format!(text, Module);
         }
